/*
 * Copyright (C) 2006 Travis Watkins.
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without
 * fee, provided that the above copyright notice appear in all copies
 * and that both that copyright notice and this permission notice
 * appear in supporting documentation, and that the name of
 * Novell, Inc. not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission.
 * Novell, Inc. makes no representations about the suitability of this
 * software for any purpose. It is provided "as is" without express or
 * implied warranty.
 *
 * NOVELL, INC. DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, 
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN
 * NO EVENT SHALL NOVELL, INC. BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
 * OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, 
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <config.h>

#include <Python.h>
#include <beryl-settings.h>

#define REGISTER_TYPE(t, n)  G_STMT_START              \
	{                                                  \
		t.ob_type = &PyType_Type;                      \
		PyType_Ready (&t);                             \
		PyModule_AddObject (mod, n, (PyObject *) &t);  \
	} G_STMT_END

typedef struct
{
	PyObject_HEAD
	BerylSettingsContext *context;
} PyBerylSettingsContext;

typedef struct
{
	PyObject_HEAD
	BerylSettingsPlugin *plugin;
} PyBerylSettingsPlugin;

typedef struct
{
	PyObject_HEAD
	BerylSetting *setting;
	BerylSettingType type;
} PyBerylSettingsSetting;

typedef struct
{
	PyObject_HEAD
	BerylSettingColorValue *color;
} PyBerylSettingsColor;

static PyTypeObject PyBerylSettingsContext_Type;
static PyTypeObject PyBerylSettingsPlugin_Type;
static PyTypeObject PyBerylSettingsSetting_Type;

static PyObject *pyberylsettings_context_create             (PyObject*, PyObject*);
static PyObject *pyberylsettings_context_new                (void);
static void      pyberylsettings_context_dealloc            (PyBerylSettingsContext*);
static PyBerylSettingsContext *pyberylsettings_context_wrap (BerylSettingsContext*);
static PyObject *pyberylsettings_context_getattro           (PyBerylSettingsContext*, PyObject*);

static PyObject *pyberylsettings_plugin_create            (PyObject*, PyObject*);
static PyObject *pyberylsettings_plugin_new               (void);
static void      pyberylsettings_plugin_dealloc           (PyBerylSettingsPlugin*);
static PyBerylSettingsPlugin *pyberylsettings_plugin_wrap (BerylSettingsPlugin*);
static PyObject *pyberylsettings_plugin_getattro          (PyBerylSettingsPlugin*, PyObject*);

static PyObject *pyberylsettings_setting_create             (PyObject*, PyObject*);
static PyObject *pyberylsettings_setting_new                (void);
static void      pyberylsettings_setting_dealloc            (PyBerylSettingsSetting*);
static PyBerylSettingsSetting *pyberylsettings_setting_wrap (BerylSetting*);
static PyObject *pyberylsettings_setting_getattro           (PyBerylSettingsSetting*, PyObject*);

static PyObject *pyberylsettings_color_create           (PyObject*, PyObject*);
static PyObject *pyberylsettings_color_new              (void);
static void      pyberylsettings_color_dealloc          (PyBerylSettingsColor*);
static PyBerylSettingsColor *pyberylsettings_color_wrap (BerylSettingColorValue*);
static PyObject *pyberylsettings_color_getattro         (PyBerylSettingsColor*, PyObject*);

void initberylsettings (void);

static PyObject *
get_py_type_for_value(BerylSettingValue *value, BerylSettingType type)
{
	gboolean success;
	gboolean bool_data;
	int int_data;
	float float_data;
	gchar *str_data;
	BerylSettingColorValue color_data;

	switch (type)
	{
	case BERYL_SETTING_TYPE_BOOL:
		success = beryl_setting_value_get_bool(value, &bool_data);
		if (success)
		{
			return PyBool_FromLong(bool_data);
		}
		break;
	case BERYL_SETTING_TYPE_INT:
		success = beryl_setting_value_get_int(value, &int_data);
		if (success)
		{
			return PyInt_FromLong(int_data);
		}
		break;
	case BERYL_SETTING_TYPE_FLOAT:
		success = beryl_setting_value_get_float(value, &float_data);
		if (success)
		{
			return PyFloat_FromDouble(float_data);
		}
		break;
	case BERYL_SETTING_TYPE_STRING:
		success = beryl_setting_value_get_bool(value, &str_data);
		if (success)
		{
			return PyString_FromString(str_data);
		}
		break;
	case BERYL_SETTING_TYPE_COLOR:
		success = beryl_setting_value_get_color(value, &color_data);
		if (success)
		{
			return pyberylsettings_color_wrap(&color_data);
		}
		break;
	}
	Py_INCREF (Py_None);
	return Py_None;
}

static PyObject *
pyberylsettings_context_find_plugin (PyObject *self, PyObject *args)
{
	PyBerylSettingsContext *context;
	BerylSettingsPlugin    *plugin;
	PyBerylSettingsPlugin  *retval;
	char *name;

	if (args != NULL)
	{
		if (!PyArg_ParseTuple (args, "s:berylsettings.Context.find_plugin", &name))
			return NULL;
    }

	context = (PyBerylSettingsContext *) self;

	if ((strcmp(name, "general")==0))
	{
		name = NULL;
	}
	plugin = beryl_settings_context_find_plugin (context->context, name);
	if (plugin == NULL)
	{
		Py_INCREF (Py_None);
		return Py_None;
	}

	retval = pyberylsettings_plugin_wrap (plugin);
	return (PyObject *) retval;
}

static PyObject *
pyberylsettings_context_get_plugins(PyBerylSettingsContext *self, PyObject *args)
{
	int c = 0;
	GSList *cur;
	PyObject *tuple;
	PyBerylSettingsPlugin *plugin;

	cur = beryl_settings_context_get_plugins(self->context);
	while (cur)
	{
		c++;
		cur = cur->next;
	}
	tuple = PyTuple_New (c);

	c = 0;
	cur = beryl_settings_context_get_plugins(self->context);
	while (cur)
	{
		plugin = pyberylsettings_plugin_wrap(cur->data);
		PyTuple_SetItem (tuple, c, (PyObject *)plugin);
		cur = cur->next;
		c++;
	}
	return tuple;
}

static void
pyberylsettings_context_write (PyBerylSettingsContext *self, PyObject *args)
{
	beryl_settings_context_write(self->context);
}

static PyObject *
pyberylsettings_context_send_reload_signal (PyBerylSettingsContext *self, PyObject *args)
{
	gboolean success;
	success = beryl_settings_send_reload_signal();
	if (success)
	{
		Py_INCREF (Py_True);
		return Py_True;
	}
	Py_INCREF (Py_False);
	return Py_False;
}

static PyObject *
pyberylsettings_plugin_get_name (PyBerylSettingsPlugin *self, PyObject *args)
{
	gchar *name = beryl_settings_plugin_get_name(self->plugin);
	if (name == NULL)
		name = "general";
	return PyString_FromString(name);
}

static PyObject *
pyberylsettings_plugin_get_short_desc (PyBerylSettingsPlugin *self, PyObject *args)
{
	gchar *short_desc = beryl_settings_plugin_get_short_desc(self->plugin);
	if (short_desc == NULL)
		short_desc = "General Settings";
	return PyString_FromString(short_desc);
}

static PyObject *
pyberylsettings_plugin_get_long_desc (PyBerylSettingsPlugin *self, PyObject *args)
{
	gchar *long_desc = beryl_settings_plugin_get_long_desc(self->plugin);
	if (long_desc == NULL)
		long_desc = "Settings for beryl as a whole";
	return PyString_FromString(long_desc);
}

static PyObject *
pyberylsettings_plugin_find_setting (PyObject *self, PyObject *args)
{
	PyBerylSettingsPlugin *plugin;
	BerylSetting    *setting;
	PyBerylSettingsSetting  *retval;
	char *name;
	gboolean is_screen;

	if (args != NULL)
	{
		if (!PyArg_ParseTuple (args, "s|i:berylsettings.Plugin.find_setting", &name, &is_screen))
			return NULL;
    }

	plugin = (PyBerylSettingsPlugin *)self;

	setting = beryl_settings_plugin_find_setting (plugin->plugin, name, is_screen);
	if (setting == NULL)
	{
		Py_INCREF (Py_None);
		return Py_None;
	}

	retval = pyberylsettings_setting_wrap (setting);
	return (PyObject *) retval;
}

static PyObject *
pyberylsettings_plugin_get_settings(PyBerylSettingsPlugin *self, PyObject *args)
{
	int c = 0;
	GSList *cur;
	PyObject *tuple;
	PyBerylSettingsSetting *setting;

	cur = beryl_settings_plugin_get_settings(self->plugin);
	while (cur)
	{
		c++;
		cur = cur->next;
	}
	tuple = PyTuple_New (c);

	c = 0;
	cur = beryl_settings_plugin_get_settings(self->plugin);
	while (cur)
	{
		setting = pyberylsettings_setting_wrap(cur->data);
		PyTuple_SetItem (tuple, c, (PyObject *)setting);
		cur = cur->next;
		c++;
	}
	return tuple;
}

static PyObject *
pyberylsettings_setting_get_name (PyBerylSettingsSetting *self, PyObject *args)
{
	gchar *name = beryl_setting_get_name(self->setting);
	return PyString_FromString(name);
}

static PyObject *
pyberylsettings_setting_get_short_desc (PyBerylSettingsSetting *self, PyObject *args)
{
	gchar *short_desc = beryl_setting_get_short_desc(self->setting);
	return PyString_FromString(short_desc);
}

static PyObject *
pyberylsettings_setting_get_long_desc (PyBerylSettingsSetting *self, PyObject *args)
{
	gchar *long_desc = beryl_setting_get_long_desc(self->setting);
	return PyString_FromString(long_desc);
}

static void
pyberylsettings_setting_reset_to_default (PyBerylSettingsSetting *self, PyObject *args)
{
	beryl_setting_reset_to_default(self->setting);
}

static PyObject *
pyberylsettings_setting_get_value(PyObject *self, PyObject *args)
{
	PyBerylSettingsSetting *setting;
	setting = (PyBerylSettingsSetting *)self;
	return get_py_type_for_value(beryl_setting_get_primary_value(setting->setting), setting->type);
}

static PyObject *
pyberylsettings_setting_get_default_value(PyObject *self, PyObject *args)
{
	BerylSettingValue *value;
	PyBerylSettingsSetting *setting;

	setting = (PyBerylSettingsSetting *)self;
	value = &(setting->setting->default_value);

	return get_py_type_for_value(value, setting->type);
}

static PyObject *
pyberylsettings_color_get_color(PyObject *self, PyObject *args)
{
	PyObject *tuple;
	BerylSettingColorValue *color;
	gdouble red, green, blue;

	color = ((PyBerylSettingsColor *)self)->color;
	red = (gdouble)color->color.red / 0xffff;
	green = (gdouble)color->color.green / 0xffff;
	blue = (gdouble)color->color.blue / 0xffff;

	tuple = PyTuple_New (3);
	PyTuple_SetItem (tuple, 0, PyLong_FromLong(red));
	PyTuple_SetItem (tuple, 1, PyLong_FromLong(green));
	PyTuple_SetItem (tuple, 2, PyLong_FromLong(blue));
	return tuple;
}

static PyObject *
pyberylsettings_color_get_alpha(PyObject *self, PyObject *args)
{
	BerylSettingColorValue *color;
	gdouble alpha;

	color = ((PyBerylSettingsColor *)self)->color;
	alpha = (gdouble)color->color.alpha / 0xffff;

	return PyLong_FromLong(alpha);
}

static PyObject *
pyberylsettings_create_context (PyObject *self, PyObject *args)
{
	BerylSettingsContext   *context;
	PyBerylSettingsContext *retval;

	if (!PyArg_ParseTuple (args, ":berylsettings.create_context"))
		return NULL;

	if (!(context = beryl_settings_context_new()))
	{
		Py_INCREF (Py_None);
		return Py_None;
	}

	beryl_settings_context_read(context);
	retval = pyberylsettings_context_wrap (context);
	return (PyObject *) retval;
}

static struct PyMethodDef pyberylsettings_context_methods[] =
{
	{"find_plugin",        (PyCFunction)pyberylsettings_context_find_plugin,        METH_VARARGS},
	{"get_plugins",        (PyCFunction)pyberylsettings_context_get_plugins,        METH_VARARGS},
	{"write",              (PyCFunction)pyberylsettings_context_write,              METH_VARARGS},
	{"send_reload_signal", (PyCFunction)pyberylsettings_context_send_reload_signal, METH_VARARGS},
	{NULL,                 NULL,                                                    0}
};

static struct PyMethodDef pyberylsettings_plugin_methods[] =
{
	{"get_name",       (PyCFunction)pyberylsettings_plugin_get_name,       METH_VARARGS},
	{"get_short_desc", (PyCFunction)pyberylsettings_plugin_get_short_desc, METH_VARARGS},
	{"get_long_desc",  (PyCFunction)pyberylsettings_plugin_get_long_desc,  METH_VARARGS},
	{"find_setting",   (PyCFunction)pyberylsettings_plugin_find_setting,   METH_VARARGS},
	{"get_settings",   (PyCFunction)pyberylsettings_plugin_get_settings,   METH_VARARGS},
	{NULL,             NULL,                                               0}
};

static struct PyMethodDef pyberylsettings_setting_methods[] =
{
	{"get_name",          (PyCFunction)pyberylsettings_setting_get_name,          METH_VARARGS},
	{"get_short_desc",    (PyCFunction)pyberylsettings_setting_get_short_desc,    METH_VARARGS},
	{"get_long_desc",     (PyCFunction)pyberylsettings_setting_get_long_desc,     METH_VARARGS},
	{"reset_to_default",  (PyCFunction)pyberylsettings_setting_reset_to_default,  METH_VARARGS},
	{"get_default_value", (PyCFunction)pyberylsettings_setting_get_default_value, METH_VARARGS},
	{"get_value",         (PyCFunction)pyberylsettings_setting_get_value,         METH_VARARGS},
	{NULL,                NULL,                                                   0}
};

static struct PyMethodDef pyberylsettings_color_methods[] =
{
	{"get_color", (PyCFunction)pyberylsettings_color_get_color, METH_VARARGS},
	{"get_alpha", (PyCFunction)pyberylsettings_color_get_alpha, METH_VARARGS},
	{NULL,        NULL,                                         0}
};

static PyMethodDef pyberylsettings_classes[] = {
	{"Context",        (PyCFunction)pyberylsettings_context_create,  0},
	{"Plugin",         (PyCFunction)pyberylsettings_plugin_create,   0},
	{"Setting",        (PyCFunction)pyberylsettings_setting_create,  0},
	{"Color",          (PyCFunction)pyberylsettings_color_create,    0},
	{"create_context", (PyCFunction)pyberylsettings_create_context,  METH_VARARGS},
	{NULL,             NULL,                                         0}
};

static PyTypeObject PyBerylSettingsContext_Type = 
{
	PyObject_HEAD_INIT(NULL) 0,                           /* ob_size */
	"Context",                              /* tp_name */
	sizeof (PyBerylSettingsContext),                      /* tp_basicsize */
	0,                                                    /* tp_itemsize */
	(destructor)pyberylsettings_context_dealloc,         /* tp_dealloc */
	(printfunc)0,
	(getattrfunc)0,
	(setattrfunc)0,
	(cmpfunc)0,
	(reprfunc)0,
	0,
	0,
	0,
	(hashfunc)0,
	(ternaryfunc)0,
	(reprfunc)0,
	(getattrofunc)pyberylsettings_context_getattro,
	(setattrofunc)0,
	0,
	Py_TPFLAGS_DEFAULT,
	NULL,
	(traverseproc)0,
	(inquiry)0,
	(richcmpfunc)0,
	0,
	(getiterfunc)0,
	(iternextfunc)0,
	pyberylsettings_context_methods
};

static PyTypeObject PyBerylSettingsPlugin_Type = 
{
	PyObject_HEAD_INIT(NULL) 0,                           /* ob_size */
	"Plugin",                               /* tp_name */
	sizeof (PyBerylSettingsPlugin),                       /* tp_basicsize */
	0,                                                    /* tp_itemsize */
	(destructor) pyberylsettings_plugin_dealloc,          /* tp_dealloc */
	0,
	(getattrfunc)0,
	(setattrfunc)0,
	(cmpfunc)0,
	(reprfunc)0,
	0,
	0,
	0,
	(hashfunc)0,
	(ternaryfunc)0,
	(reprfunc)0,
	(getattrofunc)pyberylsettings_plugin_getattro,
	(setattrofunc)0,
	0,
	Py_TPFLAGS_DEFAULT,
	NULL,
	(traverseproc)0,
	(inquiry)0,
	(richcmpfunc)0,
	0,
	(getiterfunc)0,
	(iternextfunc)0,
	pyberylsettings_plugin_methods
};

static PyTypeObject PyBerylSettingsSetting_Type = 
{
	PyObject_HEAD_INIT(NULL) 0,                           /* ob_size */
	"Setting",                               /* tp_name */
	sizeof (PyBerylSettingsSetting),                       /* tp_basicsize */
	0,                                                     /* tp_itemsize */
	(destructor) pyberylsettings_setting_dealloc,          /* tp_dealloc */
	0,
	(getattrfunc)0,
	(setattrfunc)0,
	(cmpfunc)0,
	(reprfunc)0,
	0,
	0,
	0,
	(hashfunc)0,
	(ternaryfunc)0,
	(reprfunc)0,
	(getattrofunc)pyberylsettings_setting_getattro,
	(setattrofunc)0,
	0,
	Py_TPFLAGS_DEFAULT,
	NULL,
	(traverseproc)0,
	(inquiry)0,
	(richcmpfunc)0,
	0,
	(getiterfunc)0,
	(iternextfunc)0,
	pyberylsettings_setting_methods
};

static PyTypeObject PyBerylSettingsColor_Type = 
{
	PyObject_HEAD_INIT(NULL) 0,                           /* ob_size */
	"Color",                               /* tp_name */
	sizeof (PyBerylSettingsColor),                       /* tp_basicsize */
	0,                                                     /* tp_itemsize */
	(destructor) pyberylsettings_color_dealloc,          /* tp_dealloc */
	0,
	(getattrfunc)0,
	(setattrfunc)0,
	(cmpfunc)0,
	(reprfunc)0,
	0,
	0,
	0,
	(hashfunc)0,
	(ternaryfunc)0,
	(reprfunc)0,
	(getattrofunc)pyberylsettings_color_getattro,
	(setattrofunc)0,
	0,
	Py_TPFLAGS_DEFAULT,
	NULL,
	(traverseproc)0,
	(inquiry)0,
	(richcmpfunc)0,
	0,
	(getiterfunc)0,
	(iternextfunc)0,
	pyberylsettings_color_methods
};

static PyObject *
pyberylsettings_context_create (PyObject * self, PyObject * args)
{
	if (!PyArg_NoArgs (args))
		return NULL;
	return pyberylsettings_context_new ();
}

static PyObject *
pyberylsettings_context_new ()
{
	PyBerylSettingsContext *self;
	self = PyObject_NEW (PyBerylSettingsContext, &PyBerylSettingsContext_Type);
	if (self == NULL)
		return NULL;

	self->context = NULL;
	return (PyObject *) self;
}

static void
pyberylsettings_context_dealloc (PyBerylSettingsContext *self)
{
	PyObject_DEL (self);
}

static PyBerylSettingsContext *
pyberylsettings_context_wrap (BerylSettingsContext *context)
{
	PyBerylSettingsContext *retval;
	retval = (PyBerylSettingsContext *) PyObject_NEW (PyBerylSettingsContext,
									&PyBerylSettingsContext_Type);
	if (!retval)
	    return NULL;
	retval->context = context;
	return retval;
}

static PyObject *
pyberylsettings_context_getattro (PyBerylSettingsContext *self, PyObject *py_attr)
{
	if (PyString_Check (py_attr))
	{
		char *attr;
		attr = PyString_AsString (py_attr);
		if (!strcmp (attr, "__members__"))
		{
			return Py_BuildValue ("[s]", "plugins");
		}
		else if (!strcmp (attr, "plugins"))
		{
			return (PyObject *)pyberylsettings_context_get_plugins(self, NULL);
		}
	}
	return PyObject_GenericGetAttr((PyObject *)self, py_attr);
}

static PyObject *
pyberylsettings_plugin_create (PyObject * self, PyObject * args)
{
	if (!PyArg_NoArgs (args))
		return NULL;
	return pyberylsettings_plugin_new ();
}

static PyObject *
pyberylsettings_plugin_new ()
{
	PyBerylSettingsPlugin *self;
	self = PyObject_NEW (PyBerylSettingsPlugin, &PyBerylSettingsPlugin_Type);
	if (self == NULL)
		return NULL;

	self->plugin = NULL;
	return (PyObject *) self;
}

static void
pyberylsettings_plugin_dealloc (PyBerylSettingsPlugin *self)
{
	PyObject_DEL (self);
}

static PyBerylSettingsPlugin *
pyberylsettings_plugin_wrap (BerylSettingsPlugin *plugin)
{
	PyBerylSettingsPlugin *retval;
	retval = (PyBerylSettingsPlugin *) PyObject_NEW (PyBerylSettingsPlugin,
									&PyBerylSettingsPlugin_Type);
	if (!retval)
	    return NULL;
	retval->plugin = plugin;
	return retval;
}

static PyObject *
pyberylsettings_plugin_getattro (PyBerylSettingsPlugin *self, PyObject *py_attr)
{
	if (PyString_Check (py_attr))
	{
		char *attr;
		attr = PyString_AsString (py_attr);
		if (!strcmp (attr, "__members__"))
		{
			return Py_BuildValue ("[ssss]",
				"settings", "name", "short_desc", "long_desc");
		}
		else if (!strcmp (attr, "settings"))
		{
			return (PyObject *)pyberylsettings_plugin_get_settings(self, NULL);
		}
		else if (!strcmp (attr, "name"))
		{
			return (PyObject *)pyberylsettings_plugin_get_name(self, NULL);
		}
		else if (!strcmp (attr, "short_desc"))
		{
			return (PyObject *)pyberylsettings_plugin_get_short_desc(self, NULL);
		}
		else if (!strcmp (attr, "long_desc"))
		{
			return (PyObject *)pyberylsettings_plugin_get_long_desc(self, NULL);
		}
	}
	return PyObject_GenericGetAttr((PyObject *)self, py_attr);
}

static PyObject *
pyberylsettings_setting_create (PyObject * self, PyObject * args)
{
	if (!PyArg_NoArgs (args))
		return NULL;
	return pyberylsettings_setting_new ();
}

static PyObject *
pyberylsettings_setting_new ()
{
	PyBerylSettingsSetting *self;
	self = PyObject_NEW (PyBerylSettingsSetting, &PyBerylSettingsSetting_Type);
	if (self == NULL)
		return NULL;

	self->setting = NULL;
	return (PyObject *) self;
}

static void
pyberylsettings_setting_dealloc (PyBerylSettingsSetting *self)
{
	PyObject_DEL (self);
}

static PyBerylSettingsSetting *
pyberylsettings_setting_wrap (BerylSetting *setting)
{
	PyBerylSettingsSetting *retval;
	retval = (PyBerylSettingsSetting *) PyObject_NEW (PyBerylSettingsSetting,
									&PyBerylSettingsSetting_Type);
	if (!retval)
	    return NULL;
	retval->setting = setting;
	retval->type = setting->type;
	return retval;
}

static PyObject *
pyberylsettings_setting_getattro (PyBerylSettingsSetting *self, PyObject *py_attr)
{
	if (PyString_Check (py_attr))
	{
		char *attr;
		attr = PyString_AsString (py_attr);
		if (!strcmp (attr, "__members__"))
		{
			return Py_BuildValue ("[]", "");
		}
	}
	return PyObject_GenericGetAttr((PyObject *)self, py_attr);
}

static PyObject *
pyberylsettings_color_create (PyObject * self, PyObject * args)
{
	if (!PyArg_NoArgs (args))
		return NULL;
	return pyberylsettings_color_new ();
}

static PyObject *
pyberylsettings_color_new ()
{
	PyBerylSettingsColor *self;
	self = PyObject_NEW (PyBerylSettingsColor, &PyBerylSettingsColor_Type);
	if (self == NULL)
		return NULL;

	self->color = NULL;
	return (PyObject *) self;
}

static void
pyberylsettings_color_dealloc (PyBerylSettingsColor *self)
{
	PyObject_DEL (self);
}

static PyBerylSettingsColor *
pyberylsettings_color_wrap (BerylSettingColorValue *color)
{
	PyBerylSettingsColor *retval;
	retval = (PyBerylSettingsColor *) PyObject_NEW (PyBerylSettingsColor,
									&PyBerylSettingsColor_Type);
	if (!retval)
	    return NULL;
	retval->color = color;
	return retval;
}

static PyObject *
pyberylsettings_color_getattro (PyBerylSettingsColor *self, PyObject *py_attr)
{
	if (PyString_Check (py_attr))
	{
		char *attr;
		attr = PyString_AsString (py_attr);
		if (!strcmp (attr, "__members__"))
		{
			return Py_BuildValue ("[]", "");
		}
	}
	return PyObject_GenericGetAttr((PyObject *)self, py_attr);
}

DL_EXPORT (void)
initberylsettings (void)
{
	PyObject *mod;

	mod = Py_InitModule4 ("berylsettings", pyberylsettings_classes, 0, 0, PYTHON_API_VERSION);

	REGISTER_TYPE (PyBerylSettingsContext_Type, "Context");
	REGISTER_TYPE (PyBerylSettingsPlugin_Type,  "Plugin");
	REGISTER_TYPE (PyBerylSettingsSetting_Type, "Setting");
	REGISTER_TYPE (PyBerylSettingsColor_Type, "Color");
}
